using MusiLib.Model;
using System.Diagnostics;

namespace MusiLib.Views;


public partial class Favoris : ContentPage, IAllowClick
{

    public Manager MyManager => (App.Current as App).MyManager;

    private List<Partition> partitionsInitiales;
    private List<Partition> partitionsFiltrees;
    private Trier trieur;

    public Favoris()
	{
		InitializeComponent();
        partitionsInitiales = MyManager.favoris.ToList();
        partitionsFiltrees = new List<Partition>(partitionsInitiales);
        trieur = new Trier(MyManager.favoris.ToList());
        chargerFavoris();
    }


    private void GoToAccueilByLogoButton(object sender, EventArgs e)
    {
        if (!IAllowClick.AllowTap) return;
        else IAllowClick.AllowTap = false;

        Navigation.PopAsync();

        IAllowClick.ResumeTap();
    }


    private void GoToPartitionButton(object sender, EventArgs e)
    {
        if (!IAllowClick.AllowTap) return; 
        else IAllowClick.AllowTap = false;

        var button = (ImageButton)sender;
        var idAutomation = button.AutomationId;

        if (int.TryParse(idAutomation, out int id))
        {
            Navigation.PushAsync(new PartitionView(id));
        }

        IAllowClick.ResumeTap();
    }


    private void chargerFavoris()
    {
        var imageButtonsToRemove = grille.Children.OfType<ImageButton>().Where(btn => !string.IsNullOrEmpty(btn.AutomationId)).ToList();
        foreach (var button in imageButtonsToRemove)
        {
            grille.Children.Remove(button);
        }

        int imagesParLigne = 3;
        int indice = 0;

        foreach (Partition favoris in partitionsFiltrees) 
        {
            /*int indicePartition = MyManager.partitions.IndexOf(favoris);*/

            string nomFavori = favoris.Nom;
            int indicePartition = MyManager.partitions.FindIndex(partition => partition.Nom == nomFavori);

            ImageButton imageButton = new ImageButton
            {
                Source = favoris.Image[0],
                WidthRequest = 175,
                HeightRequest = 175,
                AutomationId = indicePartition.ToString(),
            };

            imageButton.Clicked += GoToPartitionButton;

            int ligne = 1 + (indice / imagesParLigne);
            int colonne = indice % imagesParLigne;

            imageButton.Margin = GetImageButtonMargin(colonne);

            Grid.SetRow(imageButton, ligne);
            Grid.SetColumn(imageButton, colonne);
            grille.Children.Add(imageButton);

            indice++;
        }
    }

    private Thickness GetImageButtonMargin(int colonne)
    {
        if (colonne == 0)
        {
            return new Thickness(30, 0, 0, 0);
        }
        else if (colonne == 1)
        {
            return new Thickness(90, 0, 0, 0);
        }
        else
        {
            return new Thickness(150, 0, 0, 0);
        }
    }

    private void SearchBar_TextChanged(object sender, TextChangedEventArgs e)
    {
        string texteRecherche = e.NewTextValue;

        partitionsFiltrees.Clear();

        foreach (Partition partition in partitionsInitiales)
        {
            if (partition.Nom.ToLower().Contains(texteRecherche.ToLower()))
            {
                partitionsFiltrees.Add(partition);
            }
        }

        chargerFavoris();
    }

    private async void TriButton_Clicked(object sender, EventArgs e)
    {
        var action = await DisplayActionSheet("Trier par", "Annuler", null, "Type", "Difficult", "Ordre alphabtique", "Rinitialiser");

        switch (action)
        {
            case "Type":
                await TrierParTypeButton();
                break;
            case "Difficult":
                await TrierParDifficulteButton();
                break;
            case "Ordre alphabtique":
                await TrierParOrdreAlphabetiqueButton();
                break;
            case "Rinitialiser":
                ReinitialiserButton();
                break;
        }

    }


    private async Task TrierParTypeButton()
    {
        var action = await DisplayActionSheet("Trier par", "Annuler", null, "Batterie", "Guitare", "Piano");

        if (action == "Annuler")
            return;

        var partitionsTriees = trieur.TrierParInstrument(action);
        partitionsFiltrees = new List<Partition>(partitionsTriees);
        chargerFavoris();
    }

    private async Task TrierParDifficulteButton()
    {
        var action = await DisplayActionSheet("Trier par", "Annuler", null, "Facile", "Moyen", "Difficile");

        if (action == "Annuler")
            return;

        var partitionsTriees = trieur.TrierParComplexite(action);
        partitionsFiltrees = new List<Partition>(partitionsTriees);
        chargerFavoris();
    }

    private async Task TrierParOrdreAlphabetiqueButton()
    {
        var action = await DisplayActionSheet("Trier par", "Annuler", null, "Croissant", "Dcroissant");

        if (action == "Annuler")
            return;

        var partitionsTriees = trieur.TrierParOrdreAlphabetique(action);
        partitionsFiltrees = new List<Partition>(partitionsTriees);
        chargerFavoris();
    }

    private void ReinitialiserButton()
    {
        partitionsFiltrees.Clear();
        partitionsFiltrees.AddRange(partitionsInitiales);
        chargerFavoris();
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        partitionsInitiales = MyManager.favoris.ToList();
        partitionsFiltrees = new List<Partition>(partitionsInitiales);
        chargerFavoris();
    }

}